// Copyright 2013 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: jdtang@google.com (Jonathan Tang)
//
// Retrieves the title of a page.

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <menuet/os.h>

#include "gumbo.h"


static void read_file(FILE* fp, char** output, int* length) {
  int fd = fileno(fp);
  

  fseek(fp, 0, SEEK_END);
  *length = ftell(fp);
  fseek(fp, 0, SEEK_SET);  
  
  *output = malloc(*length + 1);
  int start = 0;
  int bytes_read;
  while ((bytes_read = fread(*output + start, 1, *length - start, fp))) {
    start += bytes_read;
  }
}

static const char* cleantext(const GumboNode* node) {
  //Yeah, that's not good, we should free allocated memory
  if (node->type == GUMBO_NODE_TEXT) {
    return (node->v.text.text);
  } else if (node->type == GUMBO_NODE_ELEMENT &&
             node->v.element.tag != GUMBO_TAG_SCRIPT &&
             node->v.element.tag != GUMBO_TAG_STYLE) {
    char * contents;
    int len = 4096;
    contents = malloc(len);
    *contents = "\0";
    GumboVector* children = &node->v.element.children;
    for (int i = 0; i < children->length; ++i) {
      const char * text = cleantext((GumboNode*) children->data[i]);
      if (strlen(text) + strlen(contents) + 1 > len) {
        contents = realloc(contents, len + strlen(text) + 1); //some reserve for emergency cases :P
        len = len + strlen(text) + 1;
        }
        
      if (i != 0 && strlen(text)!=0) {
        strcat(contents, " ");       
      }
      strcat(contents, text);
    }
    return contents;
  } else {
    return "";
  }
}

int main(int argc, const char** argv) {
  if (argc != 2) {
    printf("Usage: get_title <html filename>.\n");
    exit(EXIT_FAILURE);
  }
  const char* filename = argv[1];

  FILE* fp = fopen(filename, "r");
  if (!fp) {
    printf("File %s not found!\n", filename);
    exit(EXIT_FAILURE);
  }

  char* input;
  int input_length;
  read_file(fp, &input, &input_length);
  GumboOutput* output = gumbo_parse_with_options(
      &kGumboDefaultOptions, input, input_length);
  const char* title = cleantext(output->root);
  //printf("%s\n", title);
  __menuet__debug_out(title);
  __menuet__debug_out("\n");
  gumbo_destroy_output(&kGumboDefaultOptions, output);
  free(input);
}
