// Copyright 2013 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: jdtang@google.com (Jonathan Tang)
//
// Retrieves the title of a page.

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <menuet/os.h>

#include "gumbo.h"


static void read_file(FILE* fp, char** output, int* length) {
  int fd = fileno(fp);
  

  fseek(fp, 0, SEEK_END);
  *length = ftell(fp);
  fseek(fp, 0, SEEK_SET);  
  
  *output = malloc(*length + 1);
  int start = 0;
  int bytes_read;
  while ((bytes_read = fread(*output + start, 1, *length - start, fp))) {
    start += bytes_read;
  }
}

int cleantext(const GumboNode* node, int in) {
  //Yeah, that's not good, we should free allocated memory
  if (node->type == GUMBO_NODE_TEXT) {
    __menuet__debug_out("==Text:\n");
    __menuet__debug_out(node->v.text.text);
    __menuet__debug_out("==\n");
    return 0;
  } else if (node->type == GUMBO_NODE_ELEMENT &&
             node->v.element.tag != GUMBO_TAG_SCRIPT &&
             node->v.element.tag != GUMBO_TAG_STYLE) {
        
    char dbg[64];
    sprintf(dbg, "Tag code %i, depth: %i \n", node->v.element.tag, in);
    __menuet__debug_out(dbg);

    GumboVector* children = &node->v.element.children;
    for (int i = 0; i < children->length; ++i) {
      cleantext((GumboNode*) children->data[i], in+1);
    }
    return 0;
  } else {
    return 0;
  }
}

int main(int argc, const char** argv) {
  if (argc != 2) {
    printf("Usage: get_title <html filename>.\n");
    exit(EXIT_FAILURE);
  }
  const char* filename = argv[1];

  FILE* fp = fopen(filename, "r");
  if (!fp) {
    printf("File %s not found!\n", filename);
    exit(EXIT_FAILURE);
  }

  char* input;
  int input_length;
  read_file(fp, &input, &input_length);
  GumboOutput* output = gumbo_parse_with_options(
      &kGumboDefaultOptions, input, input_length);
  cleantext(output->root, 0);
  gumbo_destroy_output(&kGumboDefaultOptions, output);
  free(input);
}
